@@ -17,6 +17,7 @@
 #include <fcntl.h>
 #include <errno.h>
 #include <string.h>
+#include <malloc.h>
 #include <ctype.h>
 #include <sys/time.h>
 #include <sys/stat.h>
@@ -70,6 +71,10 @@ check_devorfile (char *file, int type)
 				if (!S_ISREG(stat_buf.st_mode))
 					return E_NOFILE;
 				break;
+			case TYPE_DIR:
+				if (!S_ISDIR(stat_buf.st_mode))
+					return E_NODIR;
+				break;
 		}
 	return 0;
 }
@@ -140,9 +145,9 @@ launch_program (char* maskstr, ...)
 						rc = E_CLDEXIT;
 				} else if (WIFSIGNALED(status))
 					rc = E_CLDSIG;
-				/* else: If the child did't exited normally and
-				 * was not terminated by a signal then it has
-				 * been stopped. In this case the lopp will
+				/* else: If the child have not exited normally
+				 * and was not terminated by a signal then it
+				 * has to be stopped. In this case the lopp will
 				 * continue and after the timeout is over the
 				 * stopped child will be killed
 				 */
@@ -287,3 +292,4 @@ axtoi (char *arg)
 	}
 	return rc;
 }
+
