@@ -37,5 +37,5 @@
 #define PEEP_TIMER    0 /* =1, timer events are printed */
 #define PEEP_NOCYCLIC 1 /* =1, some cyclic tags will be filtered out */
 
-#endif    /* INCLUDE_MODULE_CDROM_H */
+#endif    /* INCLUDE_DEBUG_H */
 
