@@ -4,7 +4,7 @@
  * module_cdrom.h
  * prototypes and definitions for the CDROM module.
  *
- * Copyright 2002 Matthias Grimm
+ * Copyright 2002-2005 Matthias Grimm
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,14 +17,16 @@
 #define DEFAULT_CDROM	"/dev/cdrom"
 
 /* public prototypes */
-int cdrom_init (struct tagitem *taglist);
+int cdrom_init ();
+int cdrom_open (struct tagitem *taglist);
+int cdrom_close ();
 int cdrom_exit ();
 
 /* private prototypes */
 void cdrom_keyboard (struct tagitem *taglist);
 void cdrom_query (struct tagitem *taglist);
 void cdrom_configure (struct tagitem *taglist);
-int cdrom_handle_tags (int cfgure, struct tagitem *taglist);
+void cdrom_handle_tags (int cfgure, struct tagitem *taglist);
 char *cdrom_getmountpoint(char *device);
 int cdrom_eject ();
 unsigned int cdrom_gettime ();
