@@ -17,12 +17,17 @@
 #include <stdio.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <string.h>
 #include "pbbinput.h"
 #include <pbb.h>
 
 #include "input_manager.h"
+#include "config_manager.h"
 #include "module_display.h"
 #include "support.h"
+#include "gettext_macros.h"
+
+#include <linux/radeonfb.h>
 
 struct moddata_display {
 	char *fbdev;         /* pathname of the framebuffer device */
@@ -47,10 +52,78 @@ struct moddata_display {
 	unsigned short modkbddn;
 	unsigned short keykbdon;
 	unsigned short modkbdon;
+	unsigned short keymirror;  /* keycode for CRT mirror key */
+	unsigned short modmirror;
 } modbase_display;
 
 int
-display_init (struct tagitem *taglist)
+display_init ()
+{
+	struct moddata_display *base = &modbase_display;
+	static char devbuffer_fb[STDBUFFERLEN];
+	int sid;
+
+	bzero (base, sizeof (struct moddata_display));
+	sprintf(devbuffer_fb, DEFAULT_FB_DEVICE);
+	base->fbdev  = devbuffer_fb;
+
+	if ((sid = registerCfgSection ("MODULE DISPLAY")) == 0) {
+		print_msg (PBB_ERR, _("Can't register configuration section %s, out of memory."), "MODULE DISPLAY");
+		return E_NOMEM;
+	} else {
+		registerCfgOptionInt (sid, "LCD_Brightness", TAG_LCDBRIGHTNESS, 0,
+				N_("initial LCD brightness level"));
+		registerCfgOptionInt (sid, "LCD_FadingSpeed", TAG_LCDFADINGSPEED, 0,
+				N_("0 = no smooth fading"));
+		registerCfgOptionBool (sid, "LCD_AutoAdjust", TAG_LCDAUTOADJUST, 0,
+				N_("only on Aluminum PowerBooks"));
+		registerCfgOptionKey (sid, "LCD_IllumUpKey", TAG_LCDILLUMUPKEY, TAG_LCDILLUMUPMOD, 0,
+				NULL);
+		registerCfgOptionKey (sid, "LCD_IllumDownKey", TAG_LCDILLUMDOWNKEY, TAG_LCDILLUMDOWNMOD, 0,
+				NULL);
+		registerCfgOptionInt (sid, "LCD_Threshold", TAG_LCDTHRESHOLD, 0,
+				N_("ambient light threshold in percent for backlight autoadj."));
+		registerCfgOptionInt (sid, "LCD_AutoAdjMin_Bat", TAG_LCDAUTOADJMINBAT, 0,
+				N_("autoadjustment range parameters"));
+		registerCfgOptionInt (sid, "LCD_AutoAdjMax_Bat", TAG_LCDAUTOADJMAXBAT, 0,
+				NULL);
+		registerCfgOptionInt (sid, "LCD_AutoAdjMin_AC", TAG_LCDAUTOADJMINAC, 0,
+				NULL);
+		registerCfgOptionInt (sid, "LCD_AutoAdjMax_AC", TAG_LCDAUTOADJMAXAC, 0,
+				NULL);
+		registerCfgOptionInt (sid, "KBD_Brightness", TAG_KBDBRIGHTNESS, 0,
+				N_("initial keyboard illumination level"));
+		registerCfgOptionInt (sid, "KBD_OnBrightness", TAG_KBDONBRIGHTNESS, 0,
+				N_("initial level if KBD on/off key is pressed"));
+		registerCfgOptionInt (sid, "KBD_FadingSpeed", TAG_KBDFADINGSPEED, 0,
+				N_("0 = no smooth fading"));
+		registerCfgOptionBool (sid, "KBD_AutoAdjust", TAG_KBDAUTOADJUST, 0,
+				N_("only on Aluminum PowerBooks"));
+		registerCfgOptionKey (sid, "KBD_IllumUpKey", TAG_KBDILLUMUPKEY, TAG_KBDILLUMUPMOD, 0,
+				NULL);
+		registerCfgOptionKey (sid, "KBD_IllumDownKey", TAG_KBDILLUMDOWNKEY, TAG_KBDILLUMDOWNMOD, 0,
+				NULL);
+		registerCfgOptionKey (sid, "KBD_IllumOnKey", TAG_KBDILLUMONKEY, TAG_KBDILLUMONMOD, 0,
+				NULL);
+		registerCfgOptionInt (sid, "KBD_Threshold", TAG_KBDTHRESHOLD, 0,
+				N_("ambient light threshold in percent for keyboard light autoadj."));
+		registerCfgOptionString (sid, "dev_FrameBuffer", TAG_FRAMEBUFFERDEVICE, 0,
+				NULL);
+		registerCfgOptionBool (sid, "UseFBBlank", TAG_BLANKFRAMEBUFFER, 0,
+				NULL);
+		registerCfgOptionBool (sid, "DimFullyDark", TAG_DIMFULLYDARK, 0,
+				NULL);
+		registerCfgOptionKey (sid, "CRT_MirrorKey", TAG_CRTMIRRORKEY, TAG_CRTMIRRORMOD, 0,
+				NULL);
+	}
+
+	register_function (QUERYQUEUE, display_query);
+	register_function (CONFIGQUEUE, display_configure);
+	return 0;
+}
+
+int
+display_open (struct tagitem *taglist)
 {
 	struct moddata_display *base = &modbase_display;
 	struct tagitem args[] = {{ TAG_BACKLIGHTMAX, 0 },
@@ -58,13 +131,9 @@ display_init (struct tagitem *taglist)
 	                         { TAG_KEYBLIGHTMAX, 0 },
 	                         { TAG_AMBIENTLIGHT, -1 },
 	                         { TAG_END, 0 }};
-	int rc, level;
-
-	static char devbuffer_fb[STDBUFFERLEN];
-	sprintf(devbuffer_fb, DEFAULT_FB_DEVICE);
+	int level;
 
 	process_queue (QUERYQUEUE, args); /* get some information from hardware module */
-	base->fbdev  = devbuffer_fb;
 
 	/* if no backlight controller was registered TAG_BACKLIGHTLEVEL
 	 * would return -1 which must be converted to be in brightness
@@ -105,6 +174,8 @@ display_init (struct tagitem *taglist)
 	base->modkbddn	= MOD_NONE;
 	base->keykbdon	= KEY_KBDILLUMTOGGLE; /* key not officially supported yet */
 	base->modkbdon	= MOD_NONE;
+	base->keymirror = KEY_F7;
+	base->modmirror = MOD_CTRL;
 
 	base->autoadj_min_bat = 1;
 	base->autoadj_max_bat = base->lcdlight.max / 2 + 1;
@@ -121,9 +192,6 @@ display_init (struct tagitem *taglist)
 	base->flags.kbd_autoadjust = 1;  /* ambient light if available       */
 	base->flags.coveropen      = 1;  /* cover open by default same as in laptop module */
 
-	if ((rc = display_handle_tags (MODE_CONFIG, taglist))) /* interprete configfile */
-		return rc;         /* critcal error occured, abort program */
-
 	/* This value must be set to 1 if the initial value of kbd_onbrightness should
 	 * take effect but this will block KBD autoadjustment mode. In KBD autoadjustement
 	 * mode the initial value makes no sense so we set the flag dependent of the
@@ -136,13 +204,17 @@ display_init (struct tagitem *taglist)
 		base->flags.kbd_switchedoff= !base->flags.kbd_autoadjust; 
 
 	register_function (KBDQUEUE, display_keyboard);
-	register_function (QUERYQUEUE, display_query);
-	register_function (CONFIGQUEUE, display_configure);
 	register_function (T10QUEUE, display_timer);
 	register_function (T1000QUEUE, display_ambienttimer);
 	return 0;
 }
 
+int
+display_close ()
+{
+	return 0;
+}
+
 int
 display_exit ()
 {
@@ -162,27 +234,49 @@ display_exit ()
  * curve for automatic brightness adjustment up or down a bit.
  * Although automatic adjustment ist enable the user keep
  * control over the brightness.
- * Functions using the shift modifier were disabled during
+ * Functions using the shift modifier is disabled during
  * automatic control.
  */
 void
 display_keyboard (struct tagitem *taglist)
 {
 	struct moddata_display *base = &modbase_display;
-	int code, value, mod, level = 0, changed, tmp;
+	int code, value, mod, level = 0, changed = 0, tmp;
 
 	code = (int) tagfind (taglist, TAG_KEYCODE, 0);
 	value = (int) tagfind (taglist, TAG_KEYREPEAT, 0);
 	mod = (int) tagfind (taglist, TAG_MODIFIER, 0);
 
 	if (value && base->flags.coveropen) {
+		/* key for video mirroring */
+		if ((code == base->keymirror) && (mod == base->modmirror))
+			if ((tmp = display_switchmirror ()) != -1)
+				singletag_to_clients(CHANGEVALUE, TAG_CRTMIRROR, tmp);
+		
+		/* keys for LCD brightness control */
 		if ((code == base->keylcdup) && ((mod & ~MOD_SHIFT) == base->modlcdup)) {
-			level = mod & MOD_SHIFT ? base->lcdlight.max : base->lcdlight.current + 1;
+			/* somebody might have changed the brightness level directly so
+			 * check if we are still synchron here
+			 */
+			display_sync_lcdbrightness ();
+			
+			/* .target and .current will normally be the same value synchronized
+			 * by the display_timer() function. We use .target here because the
+			 * powersave module might have given an order to change brightness.
+			 * In case fading was activated this order would not be fullfilled
+			 * yet (only .target has been set accordingly) and if we used
+			 * .current here we would overwrite it.
+			 */
+			level = mod & MOD_SHIFT ? base->lcdlight.max : base->lcdlight.target + 1;
 			changed = 1;
 		} else if ((code == base->keylcddn) && ((mod & ~MOD_SHIFT) == base->modlcddn)) {
-			level = mod & MOD_SHIFT ? LCD_BRIGHTNESS_MIN : base->lcdlight.current - 1;
+			display_sync_lcdbrightness ();  /* same as brightness up */
+			level = mod & MOD_SHIFT ? LCD_BRIGHTNESS_MIN : base->lcdlight.target - 1;
 			changed = 1;
-		} else if ((code == base->keykbdup) && ((mod & ~MOD_SHIFT) == base->modkbdup)) {
+		}
+		
+		/* keys for KBD illumination control */
+		if ((code == base->keykbdup) && ((mod & ~MOD_SHIFT) == base->modkbdup)) {
 			if (base->flags.kbd_switchedoff) {
 				base->kbdlight.current = base->kbdon_brightness;
 				base->flags.kbd_switchedoff = 0;
@@ -207,9 +301,7 @@ display_keyboard (struct tagitem *taglist)
 			}
 			level = display_set_kbdbrightness(level);
 			singletag_to_clients(CHANGEVALUE, TAG_KBDBRIGHTNESS, level);
-			changed = 0;
-		} else
-			changed = 0;
+		}
 
 		switch (changed) {
 		case 1:  /* LCD brightness level changed by keys */
@@ -356,11 +448,11 @@ display_configure (struct tagitem *taglist)
 	display_handle_tags (MODE_CONFIG, taglist);
 }
 
-int
+void
 display_handle_tags (int cfgure, struct tagitem *taglist)
 {
 	struct moddata_display *base = &modbase_display;
-	int rc = 0, err, level;
+	int err, level;
 
 	while (taglist->tag != TAG_END) {
 		switch (taglist->tag) {
@@ -449,10 +541,18 @@ display_handle_tags (int cfgure, struct tagitem *taglist)
 			if (cfgure)	base->modkbdon = taglist->data;
 			else		taglist->data = base->modkbdon;
 			break;
+		case TAG_CRTMIRRORKEY:
+			if (cfgure)	base->keymirror = taglist->data;
+			else		taglist->data = base->keymirror;
+			break;
+		case TAG_CRTMIRRORMOD:
+			if (cfgure)	base->modmirror = taglist->data;
+			else		taglist->data = base->modmirror;
+			break;
 		case TAG_FRAMEBUFFERDEVICE:
 			if (cfgure) {
 				if ((err = copy_path ((char *) taglist->data, base->fbdev, TYPE_CHARDEV, CPFLG_NONE)))
-					rc = tagerror (taglist, err);
+					tagerror (taglist, err);
 			} else
 				taglist->data = (long) base->fbdev;
 			break;
@@ -525,13 +625,18 @@ display_handle_tags (int cfgure, struct tagitem *taglist)
 			}
 			break;
 		case TAG_WAKEUPFROMSLEEP:  /* private tag */
-			if ((cfgure) && (base->lcdlight.current))
+			if ((cfgure) && base->lcdlight.current) {
 				display_framebuffer (1);
+			}
+			/* base->coveropen must be synchronized after sleep. This is done
+			 * by the module_pmac. This module sends the WAKEUPFROMSLEEP tag
+			 * and after a short delay the COVERSTATUS tag so that no special
+			 * actions need to be taken here.
+			 */
 			break;
 		}
 		taglist++;
 	}
-	return rc;
 }
 
 /* This functions checks the limits of a given brightness level,
@@ -583,25 +688,16 @@ display_change_brightness(int op)
 {
 	struct moddata_display *base = &modbase_display;
 
+	/* somebody might have changed the brightness level directly so
+	 * check if we are still synchron here
+	 */
+	display_sync_lcdbrightness ();
+	
 	if (((op == OP_DIM_RECOVER) && (base->flags.status == STATUS_NORMAL)) ||
 	     ((op == OP_DIM_LIGHT) && (base->flags.status != STATUS_NORMAL)) ||
 	     ((op == OP_DIM_OFF) && (base->flags.status == STATUS_OFF)))
 		return;
 
-#if 0
-	level = process_queue_single (QUERYQUEUE, TAG_BACKLIGHTLEVEL, -1));
-	if (level != base->lcdlight.current) {
-		/* Somebody else has changed the LCD brightness so update
-		 * our system. This should not really happen on a
-		 * pbbuttonsd controlled laptop so it is marked as comment
-		 * until it becomes neseccary.
-		 */
-		base->lcdlight.current = level == -1 ? 0 : level;
-		base->flags.status = STATUS_NORMAL;
-		base->flags.nobacklight = level == -1 ? 1 : 0;
-	}
-#endif
-
 	/* if a backlight controller is not available try to dim to
 	 * complete darkness in hope the framebuffer device is able
 	 * to switch the backlight off. The framebuffer option must
@@ -622,7 +718,7 @@ display_change_brightness(int op)
 		base->flags.status = STATUS_DIMMED;
 		break;
 	case OP_DIM_OFF:
-		if (!base->flags.status == STATUS_DIMMED) {
+		if (base->flags.status != STATUS_DIMMED) {
 			base->lcdlight.backup = base->lcdlight.current;
 			base->kbdlight.backup = base->kbdlight.current;
 		}
@@ -646,14 +742,62 @@ void
 display_framebuffer (int on)
 {
 	struct moddata_display *base = &modbase_display;
-	int fd = -1;
+	int fd = -1, err;
 
 	if (base->flags.ctrl_fb) {
 		if ((fd = open(base->fbdev, O_RDWR)) >= 0) {    /* open framebuffer device */
-			ioctl(fd, FBIOBLANK, !on);
+			err = ioctl(fd, FBIOBLANK, on ? 0 : VESA_POWERDOWN + 1);
+//			print_msg (PBB_INFO, "Framebuffer on: %d, error: %d\n", !on, err);
 			close (fd);
+		} else
+			print_msg (PBB_ERR, _("Can't open framebuffer device '%s'.\n"), base->fbdev);
+	}
+}
+
+void
+display_sync_lcdbrightness (void)
+{
+	struct moddata_display *base = &modbase_display;
+	int level;
+
+	level = process_queue_single (QUERYQUEUE, TAG_BACKLIGHTLEVEL, -1);
+	if (level != base->lcdlight.current) {
+		base->flags.status = STATUS_NORMAL;
+		if (level != -1) {
+			/* Somebody else has changed the LCD brightness
+			 * so sync our system. */
+			base->lcdlight.current  = level;
+			base->lcdlight.target   = level;
+			base->flags.nobacklight = 0;
+		} else {
+			/* no backlight controller available */
+			base->lcdlight.current  = base->lcdlight.max;
+			base->lcdlight.target   = base->lcdlight.max;
+			base->flags.nobacklight = 1;
 		}
 	}
 }
 
+int
+display_switchmirror ()
+{
+	struct moddata_display *base = &modbase_display;
+	unsigned long value;
+	int fd, rc = -1;
+	
+	if ((fd = open(base->fbdev, O_RDWR)) >= 0) {    /* open framebuffer device */
+		if (ioctl(fd, FBIO_ATY128_GET_MIRROR, &value) == 0) {
+			value ^= ATY_MIRROR_CRT_ON;
+			ioctl(fd, FBIO_ATY128_SET_MIRROR, &value);
+			rc = value & ATY_MIRROR_CRT_ON ? 1 : 0;
+		} else if (ioctl(fd, FBIO_RADEON_GET_MIRROR, &value) == 0) {
+			value ^= ATY_RADEON_CRT_ON;
+			ioctl(fd, FBIO_RADEON_SET_MIRROR, &value);
+			rc = value & ATY_RADEON_CRT_ON ? 1 : 0;
+		}				
+		close (fd);
+	} else
+		print_msg (PBB_ERR, _("Can't open framebuffer device '%s'.\n"), base->fbdev);
+	return rc;
+}
 
