@@ -4,7 +4,7 @@
  * module_display.h
  * prototypes and definitions for the module display.
  *
- * Copyright 2002 Matthias Grimm
+ * Copyright 2002-2005 Matthias Grimm
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -32,6 +32,19 @@
 #define FBIOBLANK	   0x4611 /* 0 or vesa-level+1 */
 #endif
 
+#ifndef VESA_POWERDOWN   /* vesa level for FBIOBLANK */
+#define VESA_POWERDOWN		3
+#endif
+
+/* Definitions to control ATI Framebuffer, because they aren't
+ * available as linux include file. Definitions for the radeon
+ * framebuffer are placed in linux/radeonfb.h
+ */
+#define ATY_MIRROR_CRT_ON	0x00000002
+#define FBIO_ATY128_GET_MIRROR	_IOR('@', 1, __u32)
+#define FBIO_ATY128_SET_MIRROR	_IOW('@', 2, __u32)
+
+
 /* operation codes (OP) for display_setlevel() */
 #define OP_DIM_LIGHT	1
 #define OP_DIM_RECOVER	2
@@ -72,7 +85,9 @@ struct display_light {
 };
 
 /* public prototypes */
-int display_init (struct tagitem *taglist);
+int display_init ();
+int display_open (struct tagitem *taglist);
+int display_close ();
 int display_exit ();
 
 /* private prototypes */
@@ -81,11 +96,13 @@ void display_query (struct tagitem *taglist);
 void display_configure (struct tagitem *taglist);
 void display_timer (struct tagitem *taglist);
 void display_ambienttimer (struct tagitem *taglist);
-int display_handle_tags (int mode, struct tagitem *taglist);
+void display_handle_tags (int mode, struct tagitem *taglist);
 int display_set_lcdbrightness(int level);
 int display_set_kbdbrightness(int level);
 int display_clip_brightness(struct display_light *illu, int level);
 void display_change_brightness(int op);
 void display_framebuffer (int on);
+void display_sync_lcdbrightness (void);
+int display_switchmirror ();
 
 #endif    /* INCLUDE_MODULE_DISPLAY_H */
