@@ -135,6 +135,7 @@ pmac_open (struct tagitem *taglist)
 {
 	struct moddata_pmac *base = &modbase_pmac;
 	char *devname;
+	long DUMMY = 0;
 	int val, rc;
 
 	base->fd_pmu		    = -1;
@@ -222,7 +223,7 @@ pmac_open (struct tagitem *taglist)
 	base->trackpad_mode = trackpad_get_config(base->fd_adb); /* get trackpad mode */
 
 	if (base->version != OHARE_PMU) {
-		ioctl(base->fd_pmu, PMU_IOC_GRAB_BACKLIGHT, 0);
+		ioctl(base->fd_pmu, PMU_IOC_GRAB_BACKLIGHT, &DUMMY);
 		base->keyboard_mode = keyboard_get_config(base->fd_adb);	/* get keyboard config*/
 	}
 
@@ -270,8 +271,15 @@ pmac_keyboard (struct tagitem *taglist)
 	mod = (int) tagfind (taglist, TAG_MODIFIER, 0);
 
 	if (value == 1 || value == 2) {
-		/* trackpad tapping control */
-		if (!isModifier (code, mod) && base->flags.notap_typing) {
+		/* trackpad tapping control
+		 * the left mouse button must be filtered out because it will
+		 * break NoTapTyping. The Trackpad will emulate a left mouse
+		 * button which causes that 'tapping' will be disabled. The
+		 * event system received a 'mouse button down' but a 'mouse
+		 * button up' will never be seen. This leads to a weird
+		 * trackpad behaviour.
+		 */
+		if (code != BTN_LEFT && !isModifier (code, mod) && base->flags.notap_typing) {
 			if (base->trackpad_pause == 0) {
 				trackpad_set_config(base->fd_adb, TRACKPAD_NOTAP);
 #if defined(DEBUG) && TRACKPAD
@@ -521,8 +529,10 @@ pmac_pmu_handler (int fd)
 			call_script ("/sbin/cardctl %s %d", "eject", intr[2]-1);
 			break;
 		case 0x40:  /* timer interrupt */
-			/* no battery present -> no timer interrupts */
+			/* no battery present -> no timer interrupts on G3 Pismo*/
+			/* n = 2 && intr[1] = 0x04 = %00100 std timer interrupt */
 			/* n = 2 && intr[1] = 0x14 = %10100 if a battery is present */
+			/* n = 2 && intr[1] = 0x0c = %01100 power button on mac-mini */
 			/* n = 6 && intr[1] = 0x1c = %11100 if power button is pressed */
 			/* n = 6 && intr[1] = 0x15 = %10101 after cover close */
 			if (n == 6 && ((intr[1] >> 3) & 1) != PBpressed) {
@@ -1069,7 +1079,8 @@ trackpad_get_dev (int fd)
 			if ((ADBBuffer[2] == 0x74) &&
 			     (ADBBuffer[3] == 0x70) &&		/* 'tpad' */
 			     (ADBBuffer[4] == 0x61) &&
-			     (ADBBuffer[5] == 0x64)) {
+			     (ADBBuffer[5] == 0x64) &&
+				 (ADBBuffer[8] == 0x03)) {     /* class:Trackpad */
 				return n;
 			}
 		}
