@@ -1,10 +1,10 @@
 #ifndef INCLUDE_MODULE_PMAC_H
 #define INCLUDE_MODULE_PMAC_H
 /* ----------------------------------------------------------------------------
- * module_PMAC.h
+ * module_pmac.h
  * prototypes and definitions for the laptop module pmac.
  *
- * Copyright 2002 Matthias Grimm
+ * Copyright 2002-2005 Matthias Grimm
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -64,8 +64,9 @@ struct modflags_pmac {
 	unsigned int coveropen :1;
 	unsigned int ac_power:1;
 	unsigned int batpresent:1;      /* set by pmu hardware */
-	unsigned int sleepsupported :1;
-	unsigned int set_tpmode:1;        /* delayed function triggered */
+	unsigned int sleepsupported:1;
+	unsigned int notap_typing:1;     /* switch trackpad tap off while typing */
+//	unsigned int set_tpmode:1;        /* delayed function triggered */
 	unsigned int set_kbdmode:1;          /* delayed function triggered */
 	unsigned int goto_sleep:1;             /* delayed function triggered */
 	unsigned int bat_present:MAX_BATTERIES; /* set by /proc/pmu/battery_% */
@@ -74,25 +75,32 @@ struct modflags_pmac {
 };
 
 /* public prototypes */
-int pmac_init (struct tagitem *taglist);
+int pmac_init ();
+int pmac_open (struct tagitem *taglist);
+int pmac_close ();
 int pmac_exit ();
 
 /* private prototypes */
 void pmac_keyboard (struct tagitem *taglist);
+void pmac_mouse (struct tagitem *taglist);
 void pmac_secure (struct tagitem *taglist);
 void pmac_query (struct tagitem *taglist);
 void pmac_configure (struct tagitem *taglist);
-int pmac_handle_tags (int cfgure, struct tagitem *taglist);
-void pmac_pmu_handler (int fd);
+void pmac_handle_tags (int cfgure, struct tagitem *taglist);
+int  pmac_pmu_handler (int fd);
 void pmac_timer (struct tagitem *taglist);
 
 void pmac_update_batteryinfo ();
 int pmac_get_procac ();
 int pmac_update_flags ();
+int pmac_getmachine ();
 void activate_sleepmode ();
 int pmac_get_lcdbacklight();
 void pmac_set_lcdbacklight(int val);
 void pmac_identify (char* buffer, int pmu);
+int isModifier (int code, int mod);
+int haveADBKeyboard ();
+int haveADBTrackpad ();
 
 int keyboard_get_config(int fd);
 void keyboard_set_config(int fd, int config);
