@@ -21,6 +21,7 @@
 #include <string.h>
 #include <signal.h>
 #include <fcntl.h>
+#include <utmp.h>
 #include <sys/time.h>
 #include <sys/ioctl.h>
 #include <sys/kd.h>
@@ -100,14 +101,14 @@ power_init ()
 				N_("'"POLICY_NONE_NAME"', '"POLICY_POWERSAVE_NAME"', '"POLICY_USER_NAME"' or '"POLICY_PERFORMANCE_NAME"'"),
 				POLICY_NONE_NAME, POLICY_POWERSAVE_NAME, POLICY_USER_NAME, POLICY_PERFORMANCE_NAME, NULL);
 		registerCfgOptionList (sid, "onAC_TimerAction", TAG_ONAC_TIMERACTION, 0,
-				N_("'"ACTION_NONE_NAME"', '"ACTION_TORAM_NAME"', '"ACTION_TODISK_NAME"' or '"ACTION_BLANK_NAME"'"),
-				ACTION_NONE_NAME, ACTION_TORAM_NAME, ACTION_TODISK_NAME, ACTION_BLANK_NAME, NULL);
+				N_("'"ACTION_NONE_NAME"', '"ACTION_TORAM_NAME"', '"ACTION_TODISK_NAME"', '"ACTION_BLANK_NAME"' or '"ACTION_SHUTDOWN_NAME"'"),
+				ACTION_NONE_NAME, ACTION_TORAM_NAME, ACTION_TODISK_NAME, ACTION_BLANK_NAME, ACTION_SHUTDOWN_NAME, NULL);
 		registerCfgOptionList (sid, "onAC_CoverAction",  TAG_ONAC_COVERACTION, 0,
 				N_("see TimerAction for possible values"),
-				ACTION_NONE_NAME, ACTION_TORAM_NAME, ACTION_TODISK_NAME, ACTION_BLANK_NAME, NULL);
+				ACTION_NONE_NAME, ACTION_TORAM_NAME, ACTION_TODISK_NAME, ACTION_BLANK_NAME, ACTION_SHUTDOWN_NAME, NULL);
 		registerCfgOptionList (sid, "onAC_KeyAction", TAG_ONAC_KEYACTION, 0,
 				N_("Action (see TimerAction) for the sleepkey"),
-				ACTION_NONE_NAME, ACTION_TORAM_NAME, ACTION_TODISK_NAME, ACTION_BLANK_NAME, NULL);
+				ACTION_NONE_NAME, ACTION_TORAM_NAME, ACTION_TODISK_NAME, ACTION_BLANK_NAME, ACTION_SHUTDOWN_NAME, NULL);
 		registerCfgOptionInt (sid, "onAC_SuspendTime", TAG_ONAC_TIMESUSPEND, 0,
 				NULL);
 		registerCfgOptionInt (sid, "onAC_DimTime",     TAG_ONAC_TIMEDIM,     0,
@@ -117,14 +118,14 @@ power_init ()
 				N_("'"POLICY_NONE_NAME"', '"POLICY_POWERSAVE_NAME"', '"POLICY_USER_NAME"' or '"POLICY_PERFORMANCE_NAME"'"),
 				POLICY_NONE_NAME, POLICY_POWERSAVE_NAME, POLICY_USER_NAME, POLICY_PERFORMANCE_NAME, NULL);
 		registerCfgOptionList (sid, "onBattery_TimerAction", TAG_ONBATT_TIMERACTION, 0,
-				N_("'"ACTION_NONE_NAME"', '"ACTION_TORAM_NAME"', '"ACTION_TODISK_NAME"' or '"ACTION_BLANK_NAME"'"),
-				ACTION_NONE_NAME, ACTION_TORAM_NAME, ACTION_TODISK_NAME, ACTION_BLANK_NAME, NULL);
+				N_("'"ACTION_NONE_NAME"', '"ACTION_TORAM_NAME"', '"ACTION_TODISK_NAME"', '"ACTION_BLANK_NAME"' or '"ACTION_SHUTDOWN_NAME"'"),
+				ACTION_NONE_NAME, ACTION_TORAM_NAME, ACTION_TODISK_NAME, ACTION_BLANK_NAME, ACTION_SHUTDOWN_NAME, NULL);
 		registerCfgOptionList (sid, "onBattery_CoverAction",  TAG_ONBATT_COVERACTION, 0,
 				N_("see TimerAction for possible values"),
-				ACTION_NONE_NAME, ACTION_TORAM_NAME, ACTION_TODISK_NAME, ACTION_BLANK_NAME, NULL);
+				ACTION_NONE_NAME, ACTION_TORAM_NAME, ACTION_TODISK_NAME, ACTION_BLANK_NAME, ACTION_SHUTDOWN_NAME, NULL);
 		registerCfgOptionList (sid, "onBattery_KeyAction", TAG_ONBATT_KEYACTION, 0,
 				N_("Action (see TimerAction) for the sleepkey"),
-				ACTION_NONE_NAME, ACTION_TORAM_NAME, ACTION_TODISK_NAME, ACTION_BLANK_NAME, NULL);
+				ACTION_NONE_NAME, ACTION_TORAM_NAME, ACTION_TODISK_NAME, ACTION_BLANK_NAME, ACTION_SHUTDOWN_NAME, NULL);
 		registerCfgOptionInt (sid, "onBattery_SuspendTime", TAG_ONBATT_TIMESUSPEND, 0,
 				NULL);
 		registerCfgOptionInt (sid, "onBattery_DimTime",     TAG_ONBATT_TIMEDIM,     0,
@@ -551,6 +552,12 @@ power_handle_tags (int cfgure, struct tagitem *taglist)
 				else if (base->mode == MODE_DIM)
 					base->sleeptime = base->activeProfile->dimtime;
 				power_check_battery (base->timeremaining, base->powersource);
+				
+				/* If the power source changed during the cover is closed
+				 * the machine should follow the active profile.
+				 */
+				if (!base->flags.coveropen)
+					power_suspend (base->activeProfile->coveraction);
 			}
 			break;
 		case TAG_TIMECHANGED:  /* private tag */
@@ -597,6 +604,10 @@ power_handle_tags (int cfgure, struct tagitem *taglist)
 			if (cfgure)	power_suspend (ACTION_TORAM);
 			else		tagerror (taglist, E_NOREAD);
 			break;
+		case TAG_GOTOHIBERNATE:
+			if (cfgure)	power_suspend (ACTION_TODISK);
+			else		tagerror (taglist, E_NOREAD);
+			break;
 		case TAG_POLICY:
 			if (cfgure) {
 				if (taglist->data == POLICY_NONE || taglist->data > POLICY_LAST)
@@ -796,9 +807,30 @@ power_check_battery (int time, int psource)
 	base->warnlevel = wl;
 }
 
-/* This function requests sleep mode and take user configurations
-    like enable_blank into account. */
+int
+power_shutdown_in_progress ()
+{
+	struct utmp *ut;
+	int runlevel=-1;
 
+	setutent();
+	while ((ut = getutent()) != NULL) {
+		if (ut->ut_type == RUN_LVL) {
+			runlevel = ut->ut_pid % 256;
+			break;
+		}
+	}
+	endutent();
+
+	if (runlevel == '0' || runlevel == '6')
+		return 1; /* shutdown or reboot in progress */
+
+	return 0;
+}
+				
+/* This function requests sleep mode and take user configurations
+ * like enable_blank into account.
+ */
 void
 power_suspend (int action)
 {
@@ -806,6 +838,12 @@ power_suspend (int action)
 
 	if (action == ACTION_NONE)  /* no action, mode will not change */
 		return;
+
+	/* If a shutdown is in progress allow only ACTION_BLANK
+	 * and reject all other actions.
+	 */
+    if (power_shutdown_in_progress() && action != ACTION_BLANK)
+		return;
 	
 	base->mode = MODE_SUSPEND;
 
@@ -817,7 +855,6 @@ power_suspend (int action)
 	case ACTION_TORAM:
 		if (base->flags.sleep_supported) {
 			singletag_to_clients(WARNING, TAG_SLEEPINSECONDS, 0);
-/*			sleep(1);    give clients a chance to react */
 			call_script (base->script_pmcs, "suspend", base->powersource ? "ac" : "battery", "ram");
 			process_queue_single (CONFIGQUEUE, TAG_REQUESTSLEEP, 0);
 			break;
@@ -829,6 +866,11 @@ power_suspend (int action)
 			power_beep();
 		}
 		break;
+	case ACTION_SHUTDOWN:  /* action currently disabled */
+		singletag_to_clients(WARNING, TAG_SLEEPINSECONDS, 0);
+		call_script (base->script_pmcs, "shutdown", base->powersource ? "ac" : "battery", "");
+		sleep (10);
+		break;
 	}
 }
 
