@@ -43,22 +43,22 @@ copy_path (char *source, char *dest, int type, int flags)
 
 		switch (rc) {
 		case E_NOEXIST:
-			print_error(_("ERROR: %s doesn't exist.\n"), source);
+			print_msg (PBB_ERR, _("The file '%s' doesn't exist.\n"), source);
 			break;
 		case E_NOFILE:
-			print_error(_("ERROR: %s is not a file.\n"), source);
+			print_msg (PBB_ERR, _("The file '%s' is not a file.\n"), source);
 			break;
 		case E_NOCHAR:
-			print_error(_("ERROR: %s is not a character device.\n"), source);
+			print_msg (PBB_ERR, _("The file '%s' is not a character device.\n"), source);
 			break;
 		case E_NOBLK:
-			print_error(_("ERROR: %s is not a block device.\n"), source);
+			print_msg (PBB_ERR, _("The file '%s' is not a block device.\n"), source);
 			break;
 		case E_USER:
-			print_error(_("SECURITY: %s must be owned by the same owner as pbbuttonsd.\n"), source);
+			print_msg (PBB_WARN, _("SECURITY: %s must be owned by the same owner as pbbuttonsd.\n"), source);
 			break;
 		case E_RIGHTS:
-			print_error(_("SECURITY: %s must be writable only be the owner of pbbuttonsd.\n"), source);
+			print_msg (PBB_WARN, _("SECURITY: %s must be writable only be the owner of pbbuttonsd.\n"), source);
 			break;
 		default:
 			if (whitespace != NULL) *whitespace = ' ';
@@ -66,7 +66,7 @@ copy_path (char *source, char *dest, int type, int flags)
 			break;
 		}
 	} else {
-		print_error(_("ERROR: Path of %s is to long.\n"), source);
+		print_msg (PBB_ERR, _("Path of '%s' is to long.\n"), source);
 		rc = E_BUFOVL;
 	}
 
@@ -117,7 +117,7 @@ call_script (char* template, ...)
 	default:
 		ftext =_("failed - unknown error code");
 	}
-	print_error(_("INFO: Script '%s' %s\n"), script, ftext);
+	print_msg (PBB_INFO, _("Script '%s' %s\n"), script, ftext);
 	return rc;
 }
 
